#!/usr/bin/env perl
#
# Creating or updating database structure
#
use warnings;
use strict;
use XAO::Web;

my $site=XAO::Web->new(sitename => '<%PROJECT%>', init_args => { check_consistency => 1 });

$site->set_current();
$site->config->build_structure();

exit(0);

# vim: ft=perl
