<%Header
  title='XAO::Web Default Template'
  keywords='xao cgi psgi apache template framework'
  description='XAO::Web Default Template'
%>

<h2>This is the default template for XAO::Web -- XAO Web Developer.</h2>

<p>
  If you see this page instead of a site then the site is probably misconfigured:
  <a href="https://ejelta.com/xao/">XAO::Web</a> engine works, but templates
  are not available.
</p>

<ul>
  <li>The time now is <em><%Date/h style="short"%></em>.</li>
  <li>The current URL is <em><%URL/h%></em>.</li>
</ul>

<p>
  Here is the source of this page in case you're interested:
</p>

<pre style="padding: 15px; box-shadow: 0 1px 9px #888">
<%Page/html path='/index.html' unparsed%>
</pre>

<%Footer%>
