<%Cookie
  mode='set'
  name='c-secure'
  value='v-secure'
  path='/'
  expires='+600s'
  secure
%>
<%Cookie
  mode='set'
  name='c-samesite'
  value='v-samesite'
  path='/'
  expires='+600s'
  secure
  samesite='Lax'
%>
